@@ -3,41 +3,9 @@ from inspect import getfullargspec
 import typing
 from typing import Callable, Union, Any, Dict
 
-import fn
-
 from amino.util.string import snake_case
 
 
-class F(fn.F):
-
-    def __truediv__(self, f):
-        return self >> (lambda a: a / f)
-
-    def __floordiv__(self, f):
-        return self >> (lambda a: a // f)
-
-    @property
-    def name(self):
-        from amino.anon import AnonCallable
-        f = (
-            self.f.func
-            if isinstance(self.f.func, AnonCallable) else
-            self.f.func.__name__
-        ) if self._is_partial else self.f.__name__
-        return str(f)
-
-    @property
-    def _is_partial(self):
-        return isinstance(self.f, partial)
-
-    def __str__(self):
-        from amino.anon import format_funcall
-        rep = (format_funcall(self.name, self.f.args, self.f.keywords)
-               if self._is_partial
-               else '{}()'.format(self.name))
-        return 'F({})'.format(rep)
-
-
 def curried(func):
     @wraps(func)
     def _curried(*args, **kwargs):
@@ -57,6 +25,9 @@ def curried(func):
 
 class Identity:
 
+    def __init__(self) -> None:
+        self.__name__ = 'identity'
+
     def __call__(self, a):
         return a
 
@@ -70,6 +41,7 @@ class Val:
 
     def __init__(self, value) -> None:
         self.value = value
+        self.__name__ = self.__class__.__name__
 
     def __call__(self):
         return self.value
@@ -94,8 +66,7 @@ def call_by_name(b: CallByName):
     return b() if callable(b) else b
 
 
-def dispatch(obj: Any, tpes: typing.List[type], prefix: str,
-             default: Callable=None) -> Any:
+def dispatch(obj: Any, tpes: typing.List[type], prefix: str, default: Callable=None) -> Any:
     @singledispatch
     def main(o, *a, **kw):
         if default is None:
