@@ -1,4 +1,4 @@
-from typing import TypeVar, Callable, Tuple
+from typing import TypeVar, Callable, Tuple, Optional, cast
 from functools import reduce
 
 from amino import maybe, List, Maybe
@@ -8,7 +8,7 @@ from amino.tc.monad import Monad
 from amino.tc.base import ImplicitInstances, tc_prop
 from amino.tc.traverse import Traverse
 from amino.tc.applicative import Applicative
-from amino.tc.foldable import Foldable
+from amino.tc.foldable import Foldable, FoldableABC
 from amino.list import flatten
 from amino.tc.zip import Zip
 from amino.tc.monoid import Monoid
@@ -45,12 +45,19 @@ class ListMonad(Monad):
 class ListTraverse(Traverse):
 
     def traverse(self, fa: List[A], f: Callable, tpe: type):
-        monad = Applicative[tpe]
+        monad = Applicative.fatal(tpe)
         def folder(z, a):
             return monad.map2(z.product(f(a)), lambda l, b: l.cat(b))
         return fa.fold_left(monad.pure(List()))(folder)
 
 
+FoldableABC.register(List)
+
+
+def _find(fa: List[A], f: Callable[[A], bool]) -> Optional[A]:
+    return next(filter(f, fa), None)
+
+
 class ListFoldable(Foldable):
 
     @tc_prop
@@ -65,7 +72,7 @@ class ListFoldable(Foldable):
         return reduce(f, fa, z)
 
     def find(self, fa: List[A], f: Callable[[A], bool]):
-        return Maybe(next(filter(f, fa), None))  # type: ignore
+        return Maybe(_find(fa, f))
 
     def find_map(self, fa: List[A], f: Callable[[A], Maybe[B]]) -> Maybe[B]:
         for el in fa:
