@@ -8,8 +8,9 @@ from fn.recur import tco
 
 from amino import Either, Right, Left, Maybe, List, Empty, __, Just, env, _
 from amino.tc.base import Implicits, ImplicitsMeta
-from amino.anon import format_funcall, lambda_str, L
+from amino.anon import L
 from amino.logging import log
+from amino.util.fun import lambda_str, format_funcall
 
 A = TypeVar('A')
 B = TypeVar('B')
@@ -52,10 +53,8 @@ class TaskException(Exception):
         from traceback import format_tb
         msg = 'Task exception{}\nCause:\n{}{}: {}\n\nCallback:\n{}'
         ex = ''.join(format_tb(self.cause.__traceback__))
-        loc = ('' if self.stack.empty else
-               ' at:\n{}'.format(self.format_stack))
-        return msg.format(loc, ex, self.cause.__class__.__name__, self.cause,
-                          self.f)
+        loc = ('' if self.stack.empty else ' at:\n{}'.format(self.format_stack))
+        return msg.format(loc, ex, self.cause.__class__.__name__, self.cause, self.f)
 
 
 class TaskMeta(ImplicitsMeta):
@@ -158,7 +157,7 @@ class Task(Generic[A], Implicits, implicits=True, metaclass=TaskMeta):
         v = self._step()
         dur = time.time() - start
         if dur > 0.1:
-            self.log.ddebug('task {} took {:.4f}s'.format(self.string, dur))
+            log.ddebug(lambda: 'task {} took {:.4f}s'.format(self.string, dur))
         return v
 
     def __repr__(self):
@@ -174,6 +173,10 @@ class Task(Generic[A], Implicits, implicits=True, metaclass=TaskMeta):
     def unsafe_perform_sync(self):
         return self.attempt
 
+    @property
+    def fatal(self) -> A:
+        return self.attempt.get_or_raise
+
     def and_then(self, nxt: 'Task[B]'):
         fs = 'and_then({})'.format(nxt.string)
         return self.flat_map(lambda a: nxt, fs=Just(fs))
@@ -233,9 +236,7 @@ class BindSuspend(Generic[A], Task[A]):
         return '{}({})'.format(self._name, self.string)
 
     def _flat_map(self, f: Callable[[A], 'Task[B]'], ts, fs) -> 'Task[B]':
-        bs = L(BindSuspend)(self.thunk,
-                            L(self.f)(_).flat_map(f, Just(ts), Just(fs)), ts,
-                            fs)
+        bs = L(BindSuspend)(self.thunk, lambda a: self.f(a).flat_map(f, Just(ts), Just(fs)), ts, fs)
         return Suspend(bs, '{}.{}'.format(ts, fs))
 
     @property
@@ -248,7 +249,6 @@ class Now(Generic[A], Task[A]):
     def __init__(self, value) -> None:
         super().__init__()
         self.value = value
-        self.string = str(self)
 
     def _step(self):
         return self
@@ -267,6 +267,10 @@ class Now(Generic[A], Task[A]):
     def fs(self):
         return 'now({})'.format(self.value)
 
+    @property
+    def string(self) -> str:
+        return str(self)
+
 
 def Try(f: Callable[..., A], *a, **kw) -> Either[Exception, A]:
     return Task.delay(f, *a, **kw).attempt
